@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -387,23 +387,27 @@ public class HttpServletRequestWrapper implements HttpServletRequest, Parameteri
         return delegate.isUserInRole(arg0);
     }
 
+    /**
+     * com.openexchange.ajax.Multiple accesses reequest objects concurrently
+     * so we have to prevent concurrent modifications
+     */
     @Override
-    public void removeAttribute(String arg0) {
+    public synchronized void removeAttribute(String arg0) {
         delegate.removeAttribute(arg0);
     }
 
     @Override
-    public void setAttribute(String arg0, Object arg1) {
+    public synchronized void setAttribute(String arg0, Object arg1) {
         delegate.setAttribute(arg0, arg1);
     }
 
     @Override
-    public void setCharacterEncoding(String arg0) throws UnsupportedEncodingException {
+    public synchronized void setCharacterEncoding(String arg0) throws UnsupportedEncodingException {
         delegate.setCharacterEncoding(arg0);
     }
 
     @Override
-    public void putParameter(String name, String value) {
+    public synchronized void putParameter(String name, String value) {
         Request internalRequest = ServletUtils.getInternalRequest(delegate);
         internalRequest.putParameter(name, value);
     }
