@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -91,6 +91,11 @@
 
 package com.openexchange.http.grizzly.service.http;
 
+import java.io.IOException;
+import java.nio.ByteBuffer;
+import java.nio.CharBuffer;
+import java.nio.charset.Charset;
+import java.nio.charset.CharsetEncoder;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Dictionary;
@@ -104,18 +109,22 @@ import java.util.TreeSet;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
+
 import javax.servlet.Filter;
 import javax.servlet.Servlet;
 import javax.servlet.ServletException;
+
 import org.glassfish.grizzly.http.server.HttpHandler;
 import org.glassfish.grizzly.http.server.Request;
 import org.glassfish.grizzly.http.server.Response;
+import org.glassfish.grizzly.http.server.io.OutputBuffer;
 import org.glassfish.grizzly.http.server.util.MappingData;
 import org.glassfish.grizzly.http.util.HttpStatus;
 import org.osgi.framework.Bundle;
 import org.osgi.service.http.HttpContext;
 import org.osgi.service.http.HttpService;
 import org.osgi.service.http.NamespaceException;
+
 import com.openexchange.exception.OXException;
 import com.openexchange.http.grizzly.servletfilter.RequestReportingFilter;
 import com.openexchange.http.grizzly.servletfilter.WrappingFilter;
@@ -140,6 +149,10 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(OSGiMainHandler.class);
 
+    private static CharBuffer reponseBuffer = CharBuffer.allocate(4096);
+
+    private static CharsetEncoder encoder = Charset.forName("UTF-8").newEncoder();
+
     private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
 
     private final Bundle bundle;
@@ -578,4 +591,35 @@ public class OSGiMainHandler extends HttpHandler implements OSGiHandler {
 
         updatePaths(request, mappingData);
     }
+
+    @Override
+    protected void customizedErrorPage(Request req, Response res)
+        throws Exception {
+
+        res.setStatus(HttpStatus.NOT_FOUND_404);
+        final ByteBuffer bb = getErrorPage(404, "Not found", "Resource does not exist.");
+        res.setContentLength(bb.limit());
+        res.setContentType("text/html");
+        OutputBuffer out = res.getOutputBuffer();
+        out.prepareCharacterEncoder();
+        out.write(bb.array(), bb.arrayOffset() + bb.position(), bb.remaining());
+        out.close();
+    }
+
+    /**
+     * Generate an error page based on our servlet defaults.
+     *
+     * @return A {@link ByteBuffer} containing the HTTP response.
+     */
+    public synchronized static ByteBuffer getErrorPage(int code,
+        String message, String description) throws IOException {
+
+        String body = com.openexchange.tools.servlet.http.Tools.getErrorPage(
+            code, message, description);
+        reponseBuffer.clear();
+        reponseBuffer.put(body);
+        reponseBuffer.flip();
+        return encoder.encode(reponseBuffer);
+
+    }
 }
