@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -73,6 +73,8 @@ public class OXRequest extends Request {
 
     private static final ThreadCache.CachedTypeIndex<Request> CACHE_IDX = ThreadCache.obtainIndex(Request.class, 16);
 
+    private static final String SESSION_COOKIE_NAME = Globals.SESSION_COOKIE_NAME;
+
     // ---------------------------- Members ---------------------------- //
 
     public static Request create() {
@@ -181,6 +183,9 @@ public class OXRequest extends Request {
                 removeInvalidSessionCookie(requestedSessionId);
             }
             if (session != null) {
+                if (isRequestedSessionIdFromURL() && !hasSessionCookie(requestedSessionId)) {
+                    response.addCookie(createSessionCookie(session.getIdInternal()));
+                }
                 return session;
             }
         }
@@ -228,9 +233,15 @@ public class OXRequest extends Request {
      * @param invalidSessionId The invalid sessionId requested by the browser/cookie
      */
     private void removeInvalidSessionCookie(String invalidSessionId) {
+        Cookie[] cookies = getCookies();
+        if (null == cookies || cookies.length <= 0) {
+            return;
+        }
+
         LOG.debug("Removing invalid JSessionId Cookie: {}", invalidSessionId);
+        String sessionCookieName = SESSION_COOKIE_NAME;
         for (Cookie cookie : cookies) {
-            if (cookie.getName().startsWith(Globals.SESSION_COOKIE_NAME)) {
+            if (cookie.getName().startsWith(sessionCookieName)) {
                 if (cookie.getValue().equals(invalidSessionId)) {
                     response.addCookie(createinvalidationCookie(cookie));
                     String domain = extractDomainValue(cookie.getValue());
@@ -297,7 +308,7 @@ public class OXRequest extends Request {
      * @return The new JSessionId Cookie
      */
     private Cookie createSessionCookie(String sessionID) {
-        Cookie jSessionIdCookie = new Cookie(Globals.SESSION_COOKIE_NAME, sessionID);
+        Cookie jSessionIdCookie = new Cookie(SESSION_COOKIE_NAME, sessionID);
 
         jSessionIdCookie.setPath("/");
 
@@ -325,6 +336,28 @@ public class OXRequest extends Request {
         return jSessionIdCookie;
     }
 
+    /**
+     * Checks if this request has a session cookie associated with specified session identifier.
+     *
+     * @param sessionID The session identifier to use for look-up
+     * @return <code>true</code> if this request contains such a session cookie; otherwise <code>false</code>
+     */
+    private boolean hasSessionCookie(String sessionID) {
+        Cookie[] cookies = getCookies();
+        if (null == cookies || cookies.length <= 0) {
+            return false;
+        }
+
+        String sessionCookieName = SESSION_COOKIE_NAME;
+        for (Cookie cookie : cookies) {
+            if (sessionCookieName.equals(cookie.getName()) && sessionID.equals(cookie.getValue())) {
+                return true;
+            }
+        }
+
+        return false;
+    }
+
     /**
      * Override isSecure by first checking the X-Forward-Proto Header. Fallback is the original implementation of the header wasn't present.
      *
